ALTER TABLE songs ADD COLUMN filetype INTEGER NOT NULL DEFAULT 0;

ALTER TABLE songs ADD COLUMN playcount INTEGER NOT NULL DEFAULT 0;

ALTER TABLE songs ADD COLUMN lastplayed INTEGER;

ALTER TABLE songs ADD COLUMN rating INTEGER;

UPDATE schema_version SET version=3;
