/* This file is part of Clementine.
   Copyright 2010, 2012, John Maguire <john.maguire@gmail.com>
   Copyright 2012, David Sansome <me@davidsansome.com>
   Copyright 2014, Krzysztof Sobiecki <sobkas@gmail.com>

   Clementine is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Clementine is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COVERS_KITTENLOADER_H_
#define COVERS_KITTENLOADER_H_

#include <QQueue>
#include <QUrl>

#include "albumcoverloader.h"

class QNetworkReply;

class KittenLoader : public AlbumCoverLoader {
  Q_OBJECT
 public:
  explicit KittenLoader(QObject* parent = nullptr);

  virtual quint64 LoadKitten(const AlbumCoverLoaderOptions& options);

 private slots:
  void KittensRetrieved(QNetworkReply* reply);
  void FetchMoreKittens();

 private:
  static const char* kFlickrKittenUrl;
  static const char* kFlickrPhotoUrl;

  QQueue<Task> pending_kittens_;
  QQueue<QUrl> kitten_urls_;
};

#endif  // COVERS_KITTENLOADER_H_
