#search, QToolButton {
  color: white;
  font-weight: bold;
  font-size: 7.5pt;
}

#search:disabled {
  color: rgba(255, 255, 255, 75%);
}

#search {
  border: 1px groove rgb(100, 100, 100);
  border-radius: 4px 4px;
  background-color: rgba(0, 0, 0, 20%);
}

QToolButton {
  font-weight: normal;
  font-weight: bold;
  color: rgba(255, 255, 255, 50%);
  border-radius: 3px;
}

QToolButton:hover {
  background-color: rgb(64, 64, 64);
  border: 1px solid rgba(255, 255, 255, 20%);
}

QToolButton:pressed {
  background-color: rgb(64, 64, 64);
  border: 1px solid rgba(255, 255, 255, 50%);
}
