/* This file is part of Clementine.
   Copyright 2010, David Sansome <me@davidsansome.com>

   Clementine is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Clementine is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libraryplaylistitem.h"

#include <QSettings>

#include "core/tagreaderclient.h"

LibraryPlaylistItem::LibraryPlaylistItem(const QString& type)
    : DbPlaylistItem(type) {}

LibraryPlaylistItem::LibraryPlaylistItem(const Song& song)
    : DbPlaylistItem("Library", song) {}

bool LibraryPlaylistItem::IsTypeSupported(const QString& type) {
  return type == "Library";
}

void LibraryPlaylistItem::Reload() {
  TagReaderClient::Instance()->ReadFileBlocking(song_.url().toLocalFile(),
                                                &song_);
}

bool LibraryPlaylistItem::InitFromQuery(const SqlRow& query) {
  // Rows from the songs tables come first
  song_.InitFromQuery(query, true);

  return song_.is_valid();
}
